package com.miqporta.cloc.cinemalocations;

import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationManager;
import android.os.AsyncTask;
import android.os.Build;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.FragmentActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.LatLngBounds;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.karumi.dexter.Dexter;
import com.karumi.dexter.PermissionToken;
import com.karumi.dexter.listener.PermissionDeniedResponse;
import com.karumi.dexter.listener.PermissionGrantedResponse;
import com.karumi.dexter.listener.PermissionRequest;
import com.karumi.dexter.listener.single.PermissionListener;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

public class MapsActivity extends FragmentActivity implements OnMapReadyCallback, View.OnClickListener,
        GoogleMap.OnInfoWindowClickListener,
        GoogleMap.OnMarkerDragListener {

    private static String TAG = "CLOC:" + MapsActivity.class.getSimpleName();

    private static int TIPO_BUSQUEDA_TITULO = 1;
    private static int TIPO_BUSQUEDA_UBICACION = 2;

    private GoogleMap mMap;
    private Map<Marker, String> allMarkersMap = new HashMap<Marker, String>();
    private Marker ultimoNuevoMarker;
    /*Objetos de la vista Botones, textos...*/
    ImageButton botonBuscar;
    ImageButton switchBuscar;
    EditText editTextBusqueda;
    FloatingActionButton botonAnyadir;
    /*Fin Objetos de la vista*/
    LocationManager locationManager;
    Location location;
    int tipoBusqueda = TIPO_BUSQUEDA_TITULO;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_maps);
        SupportMapFragment mapFragment = (SupportMapFragment) getSupportFragmentManager()
                .findFragmentById(R.id.map);
        mapFragment.getMapAsync(this);

        botonBuscar = (ImageButton) findViewById(R.id.botonBuscar);
        botonBuscar.setOnClickListener(this);
        botonAnyadir = (FloatingActionButton) findViewById(R.id.botonAnyadir);
        botonAnyadir.setOnClickListener(this);
        switchBuscar = (ImageButton) findViewById(R.id.switchBuscar);
        switchBuscar.setOnClickListener(this);
        editTextBusqueda = (EditText) findViewById(R.id.editTextBusqueda);
        editTextBusqueda.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
                if (actionId == EditorInfo.IME_ACTION_SEARCH) {
                    if (tipoBusqueda == TIPO_BUSQUEDA_TITULO) {
                        String tituloText = editTextBusqueda.getText().toString();
                        buscarPorTituloPelicula(tituloText.replace(" ", "%20"));
                    } else if (tipoBusqueda == TIPO_BUSQUEDA_UBICACION) {
                        String ubicacionText = editTextBusqueda.getText().toString();
                        buscarPorUbicacion(ubicacionText.replace(" ", "%20"));
                    }
                    ocultarTeclado(v);
                    return true;
                }
                return false;
            }
        });

    }

    /**
     * Cuando se pulsa un boton
     * @param v View donde se encuentra el boton
     */
    @Override
    public void onClick(View v) {
        Log.i(TAG, "onClick");
        switch (v.getId()) {
            case R.id.botonBuscar:
                if (tipoBusqueda == TIPO_BUSQUEDA_TITULO) {
                    String tituloText = editTextBusqueda.getText().toString();
                    buscarPorTituloPelicula(tituloText.replace(" ", "%20"));
                } else if (tipoBusqueda == TIPO_BUSQUEDA_UBICACION) {
                    String ubicacionText = editTextBusqueda.getText().toString();
                    buscarPorUbicacion(ubicacionText.replace(" ", "%20"));
                }
                ocultarTeclado(v);
                break;
            case R.id.botonAnyadir:
                addNuevoMarker();
                break;
            case R.id.switchBuscar:
                //si esta marcado titulo, y pulsamos activamos busqueda por ubicacion
                if (tipoBusqueda == TIPO_BUSQUEDA_TITULO) {
                    switchBuscar.setImageResource(R.drawable.ic_swicth_ubicacion);
                    tipoBusqueda = TIPO_BUSQUEDA_UBICACION;
                    Toast nToast = Toast.makeText(getApplicationContext(),
                            getResources().getString(R.string.mapa_switch_ubicacion), Toast.LENGTH_SHORT);
                    nToast.setGravity(Gravity.TOP, 0, 200);
                    nToast.show();

                }
                //si esta marcado ubicacion, y pulsamos activamos busqueda por titulo
                else if (tipoBusqueda == TIPO_BUSQUEDA_UBICACION) {
                    switchBuscar.setImageResource(R.drawable.ic_switch_titulo);
                    tipoBusqueda = TIPO_BUSQUEDA_TITULO;
                    Toast nToast = Toast.makeText(getApplicationContext(),
                            getResources().getString(R.string.mapa_switch_pelicula), Toast.LENGTH_SHORT);
                    nToast.setGravity(Gravity.TOP, 0, 200);
                    nToast.show();
                }
                break;
            default:
                break;
        }
    }

    @Override
    public void onInfoWindowClick(Marker marker) {
        Log.i(TAG, "onInfoWindowClick");

        String idEscena = allMarkersMap.get(marker);

        //borramos el nuevo marcador del mapa
        if ("Nuevo".equals(idEscena)) {
            marker.remove();
        }

        //abrimos nueva activity
        Intent myIntent = new Intent(this, DetailActivity.class);
        myIntent.putExtra("escenaId", idEscena);
        myIntent.putExtra("latitud", Double.toString(marker.getPosition().latitude));
        myIntent.putExtra("longitud", Double.toString(marker.getPosition().longitude));
        startActivity(myIntent);
    }


    /*Implementamos los metodos de drag de los Markers sin modificarlos, porque lo tenemos activado*/
    @Override
    public void onMarkerDragStart(Marker marker) {
    }

    @Override
    public void onMarkerDrag(Marker marker) {
    }

    @Override
    public void onMarkerDragEnd(Marker marker) {
    }

    /**
     * Cuando el mapa esta listo
     * @param googleMap Mapa instanciado
     */
    @Override
    public void onMapReady(GoogleMap googleMap) {
        mMap = googleMap;

        // Controles UI
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            Dexter.withActivity(this)
                    .withPermission(android.Manifest.permission.ACCESS_FINE_LOCATION)
                    .withListener(new PermissionListener() {
                        @Override public void onPermissionGranted(PermissionGrantedResponse response) {/* ... */}
                        @Override public void onPermissionDenied(PermissionDeniedResponse response) {/* ... */}
                        @Override public void onPermissionRationaleShouldBeShown(PermissionRequest permission, PermissionToken token) {/* ... */}
                    }).check();
        }

        if (ActivityCompat.checkSelfPermission(this, android.Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED) {
            return;
        }
        mMap.setMyLocationEnabled(true);


        locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);

        location = locationManager.getLastKnownLocation(locationManager.GPS_PROVIDER);

        if(location != null){
            String url = getResources().getString(R.string.conf_clocWs_dominio)
                    + getResources().getString(R.string.conf_clocWs_buscarPorUbicacion)
                    + Double.toString(location.getLatitude()) + "/" + Double.toString(location.getLongitude());
            new buscarLocalizacionesTask().execute(url);
        }

        mMap.setOnInfoWindowClickListener(this);
        mMap.setOnMarkerDragListener(this);

        if(location != null) {
            mMap.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(location.getLatitude(), location.getLongitude()), 8));
        }
    }

    /**
     * Encargado de ejecutar tarea asincrona buscarlocalizaciones
     * @param titulo Titulo de la pelicula
     */
    public void buscarPorTituloPelicula(String titulo){
        if(titulo != null && !titulo.equals("")) {
            String url = getResources().getString(R.string.conf_clocWs_dominio)
                    + getResources().getString(R.string.conf_clocWs_buscarPorPelicula)
                    + titulo;
            new buscarLocalizacionesTask().execute(url);
        }else{
            Toast nToast = Toast.makeText(getApplicationContext(),
                    getResources().getString(R.string.mapa_no_resultados), Toast.LENGTH_SHORT);
            nToast.setGravity(Gravity.TOP, 0,200);
            nToast.show();
        }
    }

    /**
     * Encargado de ejecutar tarea asincrona buscarCoorndenadasPorNombreLugar, esta ejecutara buscarlocalizaciones
     * @param ubicacion Lugar, ciudad, calle...
     */
    public void buscarPorUbicacion(String ubicacion){
        if(ubicacion != null && !ubicacion.equals("")) {
            String url =  getResources().getString(R.string.conf_googleMapsWs_dominio)
                            + getResources().getString(R.string.conf_googleMapsWs_buscarPorUbicacion)
                            + ubicacion;
            new buscarCoordenadasPorNombreLugarTask().execute(url);
        }else {
            Toast nToast = Toast.makeText(getApplicationContext(),
                    getResources().getString(R.string.mapa_no_resultados_ubicacion), Toast.LENGTH_SHORT);
            nToast.setGravity(Gravity.TOP, 0, 200);
            nToast.show();
        }
    }

    private class buscarLocalizacionesTask extends AsyncTask<String, Void, String> {

        String output;
        StringBuilder respuesta = new StringBuilder();

        @Override
        protected void onPreExecute () {
            Log.i(TAG, "onPreExecute buscarLocalizacionesTask");
            //Quitar Markers Existentes
            mMap.clear();
            allMarkersMap.clear();
        }

        @Override
        protected String doInBackground (String...params){
            Log.i(TAG, "doInBackground buscarLocalizacionesTask");

            try {
                String metodoGet = params[0];

                URL url = new URL(metodoGet);
                HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                conn.setRequestMethod("GET");
                conn.setRequestProperty("Accept", "application/json");

                if (conn.getResponseCode() != HttpURLConnection.HTTP_OK) {
                    throw new RuntimeException("Failed : HTTP error code : "
                            + conn.getResponseCode());
                }

                BufferedReader br = new BufferedReader(new InputStreamReader(
                        (conn.getInputStream())));


                while ((output = br.readLine()) != null) {
                    respuesta.append(output);
                }

                conn.disconnect();

            } catch (ProtocolException e) {
                e.printStackTrace();
            } catch (MalformedURLException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }

            return respuesta.toString();
        }

        @Override
        protected void onPostExecute (String respuesta){
            Log.i(TAG, "onPostExecute buscarLocalizacionesTask");
            Log.i(TAG, respuesta + "");

              try {
                JSONObject jsnobject = new JSONObject(respuesta);
                JSONArray jsonArray = jsnobject.getJSONArray("localizaciones");
                JSONObject localizacionMapa;
                LatLngBounds.Builder builder = new LatLngBounds.Builder();

                for (int i = 0; i < jsonArray.length(); i++) {
                    localizacionMapa = jsonArray.getJSONObject(i);

                    Marker marker = mMap.addMarker(new MarkerOptions()
                            .position(new LatLng(localizacionMapa.getDouble("latitud"),
                                    localizacionMapa.getDouble("longitud")))
                            .title(localizacionMapa.getString("tituloVO")));

                    //anyadimos el marker en un HashMap con su respectivo identificador
                    allMarkersMap.put(marker, localizacionMapa.getString("escenaId"));

                    //servira para calcular los limites del mapa
                    builder.include(marker.getPosition());

                }

                //si hay resultados movemos la camara
                if (jsonArray.length() > 0) {
                    //movemos la camara segun la posiscion de los markers
                    LatLngBounds bounds = builder.build();
                    int padding = 90; // offset from edges of the map in pixels
                    CameraUpdate cu = CameraUpdateFactory.newLatLngBounds(bounds, padding);
                    mMap.animateCamera(cu);
                } else {
                    Toast nToast = Toast.makeText(getApplicationContext(),
                            getResources().getString(R.string.mapa_no_resultados), Toast.LENGTH_SHORT);
                    nToast.setGravity(Gravity.TOP, 0, 200);
                    nToast.show();
                }

            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

    }

    private class buscarCoordenadasPorNombreLugarTask extends AsyncTask<String, Void, String> {

        String output;
        StringBuilder respuesta = new StringBuilder();

        @Override
        protected void onPreExecute (){
            Log.i(TAG, "onPreExecute buscarCoordenadasPorNombreLugarTask");
        }

        @Override
        protected String doInBackground(String... params) {
            Log.i(TAG, "doInBackground buscarCoordenadasPorNombreLugarTask");
            try {
                String metodoGet = params[0];

                URL url = new URL(metodoGet);
                HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                conn.setRequestMethod("GET");
                conn.setRequestProperty("Accept", "application/json");

                if (conn.getResponseCode() != HttpURLConnection.HTTP_OK) {
                    throw new RuntimeException("Failed : HTTP error code : "
                            + conn.getResponseCode());
                }

                BufferedReader br = new BufferedReader(new InputStreamReader(
                        (conn.getInputStream())));

                while ((output = br.readLine()) != null) {
                    respuesta.append(output);
                }

                conn.disconnect();

            } catch (ProtocolException e) {
                e.printStackTrace();
            } catch (MalformedURLException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }

            return respuesta.toString();
        }

        @Override
        protected void onPostExecute (String respuesta){
            Log.i(TAG, "onPostExecute buscarCoordenadasPorNombreLugarTask");
            String latitud = "";
            String longitud = "";

            try {
                JSONObject status = new JSONObject(respuesta);

                if(!status.getString("status").equals("ZERO_RESULTS")) {

                    JSONObject jsonObject = new JSONObject(respuesta);
                    JSONArray jsonArrayResults = jsonObject.getJSONArray("results");
                    latitud = jsonArrayResults.getJSONObject(0).getJSONObject("geometry")
                            .getJSONObject("location").getString("lat");
                    longitud = jsonArrayResults.getJSONObject(0).getJSONObject("geometry")
                            .getJSONObject("location").getString("lng");


                    String url =  getResources().getString(R.string.conf_clocWs_dominio)
                            + getResources().getString(R.string.conf_clocWs_buscarPorUbicacion)
                            + latitud + "/" +longitud;
                    new buscarLocalizacionesTask().execute(url);

                }else{
                    Toast nToast = Toast.makeText(getApplicationContext(),
                            getResources().getString(R.string.mapa_no_resultados_ubicacion), Toast.LENGTH_SHORT);
                    nToast.setGravity(Gravity.TOP, 0, 200);
                    nToast.show();
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    }

    private void addNuevoMarker(){

        //si ya existia un marker para añadir, lo borramos
        if(ultimoNuevoMarker != null){
            ultimoNuevoMarker.remove();
        }

        Marker marker = mMap.addMarker(new MarkerOptions()
                .position(mMap.getCameraPosition().target)
                .title(getResources().getString(R.string.mapa_markerInfo_anyadir))
                .snippet(getResources().getString(R.string.mapa_markerInfo_anyadir_snippet))
                .draggable(true)
                .icon(BitmapDescriptorFactory.fromResource(R.drawable.ic_blue_marker)));

        allMarkersMap.put(marker,"Nuevo");
        ultimoNuevoMarker = marker;

        marker.showInfoWindow();
        mMap.animateCamera(CameraUpdateFactory.newLatLng(marker.getPosition()), 250, null);

    }

    public void ocultarTeclado(View v){
        // Ocultar teclado virtual
        InputMethodManager imm =
                (InputMethodManager) getSystemService(INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(v.getWindowToken(), 0);
    }

}
